@@ -8,7 +8,10 @@
  */
 
 #include <config.h>
+
 #include <stdio.h>
+#include <math.h>
+
 #include <gtk/gtksignal.h>
 #include <gtk/gtkmarshal.h>
 #include <gtk/gtktypeutils.h>
@@ -20,7 +23,7 @@
 #include <ggvutils.h>
 
 struct _GgvControlPrivate {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 
 	BonoboZoomable *zoomable;
 	float zoom_level;
@@ -34,9 +37,25 @@ struct _GgvControlPrivate {
 };
 
 struct _GgvControlClassPrivate {
+#ifdef DYNAMIC_ZOOM_MENUS
 	BonoboUINode *zoom_level_menus;
+#endif /* DYNAMIC_ZOOM_MENUS */
 };
 
+#ifndef DYNAMIC_ZOOM_MENUS
+static struct {
+	gfloat level;
+	gchar *path;
+} zoom_level_items[] = {
+	{ 1.0/4.0, "/commands/Zoom0104" },
+	{ 1.0/2.0, "/commands/Zoom0102" },
+	{ 1.0/1.0, "/commands/Zoom0101" },
+	{ 2.0/1.0, "/commands/Zoom0201" },
+	{ 4.0/1.0, "/commands/Zoom0401" },
+	{ 0.0, NULL }
+};
+#endif /* !DYNAMIC_ZOOM_MENUS */
+
 static BonoboControlClass *ggv_control_parent_class;
 
 static void
@@ -82,6 +101,24 @@ ggv_control_finalize (GObject *object)
 	G_OBJECT_CLASS (ggv_control_parent_class)->finalize (object);
 }
 
+static void
+sync_zoom_level_items(GgvControl *control)
+{
+	int i;
+
+	for(i = 0; zoom_level_items[i].path; i++) {
+		if(fabs(ggv_postscript_view_get_zoom_factor(control->priv->ps_view) - zoom_level_items[i].level) < 0.001) {
+			bonobo_ui_component_set_prop(control->priv->uic,
+										 zoom_level_items[i].path,
+										 "state", "1", NULL);
+			return;
+		}
+	}
+	bonobo_ui_component_set_prop(control->priv->uic,
+								 "/commands/ZoomOther",
+								 "state", "1", NULL);
+}
+
 static void
 zoomable_set_frame_cb (BonoboZoomable *zoomable, GgvControl *control)
 {
@@ -93,11 +130,14 @@ zoomable_set_frame_cb (BonoboZoomable *zoomable, GgvControl *control)
 
 static void
 zoomable_set_zoom_level_cb (BonoboZoomable *zoomable, float new_zoom_level,
-			    GgvControl *control)
+							GgvControl *control)
 {
 	g_return_if_fail (control != NULL);
 	g_return_if_fail (GGV_IS_CONTROL (control));
 
+	if(fabs(ggv_postscript_view_get_zoom_factor(control->priv->ps_view) - new_zoom_level) < 0.001)
+		return;
+
 	ggv_postscript_view_set_zoom_factor
 		(control->priv->ps_view, new_zoom_level);
 	control->priv->zoom_level = ggv_postscript_view_get_zoom_factor
@@ -105,6 +145,8 @@ zoomable_set_zoom_level_cb (BonoboZoomable *zoomable, float new_zoom_level,
 
 	bonobo_zoomable_report_zoom_level_changed
 		(zoomable, control->priv->zoom_level, NULL);
+
+	sync_zoom_level_items(control);
 }
 
 static void
@@ -158,6 +200,8 @@ zoomable_zoom_to_fit_cb (BonoboZoomable *zoomable, GgvControl *control)
 	new_zoom_level =
 		ggv_postscript_view_zoom_to_fit (control->priv->ps_view, FALSE);
 
+	g_warning("zoom to fit: %f", new_zoom_level);
+
 	g_signal_emit_by_name (G_OBJECT (zoomable), "set_zoom_level",
 						   new_zoom_level);
 }
@@ -195,6 +239,51 @@ scrollbar_button_release_event(GtkWidget *widget, GdkEventButton *event,
 	return FALSE;
 }
 
+static void
+listener_ZoomLevel_cb(BonoboUIComponent *uic, const char *path,
+					  Bonobo_UIComponent_EventType type, const char *state,
+					  gpointer user_data)
+{
+	BonoboZoomable *zoomable;
+	GgvControl *control;
+	const char *zl;
+
+	g_return_if_fail(user_data != NULL);
+	g_return_if_fail(GGV_IS_CONTROL(user_data));
+
+	if(type != Bonobo_UIComponent_STATE_CHANGED)
+		return;
+
+	if(!state || !atoi(state))
+		return;
+
+	control = GGV_CONTROL(user_data);
+	zoomable = control->priv->zoomable;
+
+	zl = path + strlen("Zoom");
+
+	if(!strcmp(zl, "0104")) {
+		g_signal_emit_by_name (G_OBJECT (zoomable), "set_zoom_level",
+							   ((gfloat)1.0/4.0));
+	}
+	else if(!strcmp(zl, "0102")) {
+		g_signal_emit_by_name (G_OBJECT (zoomable), "set_zoom_level",
+							   ((gfloat)1.0/2.0));
+	}
+	else if(!strcmp(zl, "0101")) {
+		g_signal_emit_by_name (G_OBJECT (zoomable), "set_zoom_level",
+							   ((gfloat)1.0));
+	}
+	else if(!strcmp(zl, "0201")) {
+		g_signal_emit_by_name (G_OBJECT (zoomable), "set_zoom_level",
+							   ((gfloat)2.0/1.0));
+	}
+	else if(!strcmp(zl, "0401")) {
+		g_signal_emit_by_name (G_OBJECT (zoomable), "set_zoom_level",
+							   ((gfloat)4.0/1.0));
+	}
+}
+
 static gboolean
 ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 							gpointer data)
@@ -203,7 +292,7 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 	gint key = event->keyval;
 	GgvControl *control = GGV_CONTROL(data);
 	GtkGSOrientation orientation = gtk_gs_get_orientation(gs);
-	GgvPostscriptView *ps_view = control->priv->ps_view;
+	GgvPostScriptView *ps_view = control->priv->ps_view;
 
 	/*ugh. the possibilities! */
 	switch (key) {
@@ -295,6 +384,21 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 	return TRUE;
 }
 
+static void
+verb_ZoomLevel_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname)
+{
+	GgvControl *control;
+
+	g_return_if_fail (user_data != NULL);
+	g_return_if_fail (GGV_IS_CONTROL (user_data));
+
+	control = GGV_CONTROL (user_data);
+
+	g_signal_emit_by_name (G_OBJECT (control->priv->zoomable),
+						   "zoom_in");
+}
+
+
 static void
 verb_ZoomIn_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname)
 {
@@ -353,6 +457,23 @@ verb_ZoomToFit_cb (BonoboUIComponent *uic, gpointer user_data,
 						   "zoom_to_fit");
 }
 
+static void
+set_zoom_items_sensitivity(GgvControl *control, gboolean sens)
+{
+	gchar *prop_val = sens?"1":"0";
+
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/ZoomIn",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/ZoomOut",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/ZoomToDefault",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/ZoomToFit",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/menu/ViewPlaceholder/View/ZoomMenu/Zoom",
+								 "sensitive", prop_val, NULL);
+}
+
 static BonoboUIVerb ggv_control_verbs[] = {
 		BONOBO_UI_VERB ("ZoomIn",        verb_ZoomIn_cb),
 		BONOBO_UI_VERB ("ZoomOut",       verb_ZoomOut_cb),
@@ -364,23 +485,37 @@ static BonoboUIVerb ggv_control_verbs[] = {
 static void
 ggv_control_create_ui (GgvControl *control)
 {
+#ifdef DYNAMIC_ZOOM_MENUS
 	BonoboUINode *zoom_level_menus;
+#endif /* DYNAMIC_ZOOM_MENUS */
 	GgvControlClass *klass;
+	int i;
 
 	g_return_if_fail (control != NULL);
 	g_return_if_fail (GGV_IS_CONTROL (control));
 
 	klass = GGV_CONTROL_CLASS(G_OBJECT_GET_CLASS(control));
 
-
 	bonobo_ui_util_set_ui(control->priv->uic, NULL,
 						  "ggv-control-ui.xml", "GGV", NULL);
 
+#ifdef DYNAMIC_ZOOM_MENUS
 	zoom_level_menus = bonobo_ui_node_copy(klass->priv->zoom_level_menus,
 										   TRUE);
 	bonobo_ui_component_set_tree(control->priv->uic,
 								 "/menu/ViewPlaceholder/View/ZoomMenu/Zoom/ZoomLevels",
 								 zoom_level_menus, NULL);
+#else
+	for(i = 0; zoom_level_items[i].path; i++)
+		bonobo_ui_component_add_listener(control->priv->uic,
+										 zoom_level_items[i].path + strlen("/commands/"),
+										 listener_ZoomLevel_cb, control);
+	
+#endif /* DYNAMIC_ZOOM_MENUS */
+
+	set_zoom_items_sensitivity(control, ggv_postscript_view_get_page_count(control->priv->ps_view) > 0);
+
+	sync_zoom_level_items(control);
 
 	bonobo_ui_component_add_verb_list_with_data
 		(control->priv->uic, ggv_control_verbs,
@@ -459,6 +594,7 @@ ggv_control_class_init (GgvControlClass *klass)
 
 	klass->priv = g_new0(GgvControlClassPrivate, 1);
 
+#ifdef DYNAMIC_ZOOM_MENUS
 	placeholder = bonobo_ui_node_new("placeholder");
 	bonobo_ui_node_set_attr(placeholder, "name", "ZoomLevels");
 	for(i = 0; i <= ggv_max_zoom_levels; i++) {
@@ -471,6 +607,7 @@ ggv_control_class_init (GgvControlClass *klass)
 		bonobo_ui_node_add_child(placeholder, item);
 	}
 	klass->priv->zoom_level_menus = placeholder;
+#endif /* DYNAMIC_ZOOM_MENUS */
 }
 
 static void
@@ -481,13 +618,28 @@ ggv_control_init (GgvControl *control)
 
 BONOBO_TYPE_FUNC (GgvControl, BONOBO_TYPE_CONTROL, ggv_control);
 
+static void
+ps_view_property_changed_handler(BonoboListener    *listener,
+                                 char              *event_name, 
+                                 CORBA_any         *any,
+                                 CORBA_Environment *ev,
+                                 gpointer           data)
+{
+        GgvControl *control = GGV_CONTROL(data);
+
+        if(!g_strcasecmp(event_name, "Bonobo/Property:change:page")) {
+			set_zoom_items_sensitivity(control, BONOBO_ARG_GET_INT(any) != -1);
+        }
+}
+
 GgvControl *
-ggv_control_construct (GgvControl *control, GgvPostscriptView *ps_view)
+ggv_control_construct (GgvControl *control, GgvPostScriptView *ps_view)
 {
 	BonoboControl         *retval;
 	BonoboPropertyBag     *property_bag;
 	BonoboPropertyControl *property_control;
 	GtkWidget *view;
+	gchar *mask;
 
 	g_return_val_if_fail (ps_view != NULL, NULL);
 	g_return_val_if_fail (control != NULL, NULL);
@@ -563,7 +715,7 @@ ggv_control_construct (GgvControl *control, GgvPostscriptView *ps_view)
 					  G_CALLBACK (zoomable_zoom_to_default_cb),
 					  control);
 
-	control->priv->zoom_level = 1.0;
+	control->priv->zoom_level = ggv_postscript_view_get_zoom_factor(ps_view);
 	bonobo_zoomable_set_parameters_full (control->priv->zoomable,
 										 control->priv->zoom_level,
 										 ggv_zoom_levels [0],
@@ -578,8 +730,13 @@ ggv_control_construct (GgvControl *control, GgvPostscriptView *ps_view)
 
 	property_bag =
 		ggv_postscript_view_get_property_bag (control->priv->ps_view);
-	bonobo_control_set_properties (BONOBO_CONTROL (control),
+
+	bonobo_control_set_properties (BONOBO_CONTROL(control),
 								   BONOBO_OBJREF(property_bag), NULL);
+	mask =  "Bonobo/Property:change:page";
+	bonobo_event_source_client_add_listener(BONOBO_OBJREF(property_bag),
+											(BonoboListenerCallbackFn)ps_view_property_changed_handler,
+											mask, NULL, control);
 	bonobo_object_unref (BONOBO_OBJECT (property_bag));
 
 	property_control =
@@ -599,11 +756,13 @@ ggv_control_construct (GgvControl *control, GgvPostscriptView *ps_view)
 							(GConfClientNotifyFunc)ggv_control_prefs_changed,
 							control, NULL, NULL);
 
+	set_zoom_items_sensitivity(control, FALSE);
+
 	return control;
 }
 
 GgvControl *
-ggv_control_new (GgvPostscriptView *ps_view)
+ggv_control_new (GgvPostScriptView *ps_view)
 {
 	GgvControl *control;
 	
