@@ -39,9 +39,9 @@ struct _GgvControlClass {
 };
 
 GtkType        ggv_control_get_type  (void);
-GgvControl    *ggv_control_new       (GgvPostscriptView *ps_view);
+GgvControl    *ggv_control_new       (GgvPostScriptView *ps_view);
 GgvControl    *ggv_control_construct (GgvControl *control,
-				      GgvPostscriptView *ps_view);
+				      GgvPostScriptView *ps_view);
 
 G_END_DECLS
 
