@@ -26,7 +26,9 @@
 
 #include <bonobo.h>
 
-struct _GgvPostscriptViewPrivate {
+#undef DYNAMIC_PAPER_MENUS
+
+struct _GgvPostScriptViewPrivate {
 	GtkWidget *gs;
 	GtkObject *hadj, *vadj;
 
@@ -46,9 +48,11 @@ struct _GgvPostscriptViewPrivate {
 	gboolean               zoom_fit;
 };
 
-struct _GgvPostscriptViewClassPrivate {
+struct _GgvPostScriptViewClassPrivate {
+#ifdef DYNAMIC_PAPER_MENUS
 	BonoboUINode   *size_menus;
 	gchar          **size_paths;
+#endif /* DYNAMIC_PAPER_MENUS */
 };
 
 enum {
@@ -77,7 +81,7 @@ static GdkCursor *pan_cursor = NULL;
 
 static BonoboObjectClass *ggv_postscript_view_parent_class;
 
-static gchar *orientation_paths[] = {
+static const gchar *orientation_paths[] = {
 	"/commands/OverrideOrientation",
 	"/commands/OrientationPortrait",
 	"/commands/OrientationLandscape",
@@ -86,60 +90,30 @@ static gchar *orientation_paths[] = {
 	NULL
 };
 
-static void
-set_page_item_sensitivity(GgvPostscriptView *ps_view)
-{
-	GtkGS *gs = GTK_GS(ps_view->priv->gs);
-	gchar *prop_val;
-
-	prop_val = (gtk_gs_get_current_page(gs) >= gtk_gs_get_page_count(gs) - 1)?
-		"0":"1";
-	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/NextPage",
-								 "sensitive", prop_val, NULL);
-	prop_val = (gtk_gs_get_current_page(gs) <= 0 || !gs->structured_doc)?
-		"0":"1";
-	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/PrevPage",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/FirstPage",
-								 "sensitive", prop_val, NULL);
-	prop_val = (gtk_gs_get_current_page(gs) >= gtk_gs_get_page_count(gs) - 1 ||
-				!gs->structured_doc)?
-		"0":"1";
-	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/LastPage",
-								 "sensitive", prop_val, NULL);
-
-}
-
-static void
-notify_page_change(GgvPostscriptView *ps_view)
-{
-	BonoboArg *arg;
-
-	if(gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs)) < 0)
-		return;
-
-	arg = bonobo_arg_new(TC_CORBA_long);
-	*(CORBA_long *)arg->_value = gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
-	bonobo_event_source_notify_listeners(ps_view->priv->property_bag->es,
-										 "Bonobo/Property:change:page",
-										 arg, NULL);
-	bonobo_arg_release(arg);
-}
+#ifndef DYNAMIC_PAPER_MENUS
+static const gchar *size_paths[] = {
+	"/commands/OverrideSize",
+	"/commands/PaperSizeBBox",
+	"/commands/PaperSizeLetter",
+	"/commands/PaperSizeTabloid",
+	"/commands/PaperSizeLedger",
+	"/commands/PaperSizeLegal",
+	"/commands/PaperSizeStatement",
+	"/commands/PaperSizeExecutive",
+	"/commands/PaperSizeA3",
+	"/commands/PaperSizeA4",
+	"/commands/PaperSizeA5",
+	"/commands/PaperSizeB4",
+	"/commands/PaperSizeB5",
+	"/commands/PaperSizeFolio",
+	"/commands/PaperSizeQuarto",
+	"/commands/PaperSize10x14",
+	NULL
+};
+#endif /* DYNAMIC_PAPER_MENUS */
 
 void
-ggv_postscript_view_goto_page(GgvPostscriptView *ps_view, gint page)
-{
-	gint old_page = gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
-
-	gtk_gs_goto_page(GTK_GS(ps_view->priv->gs), page);
-	if(gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs)) != old_page) {
-		notify_page_change(ps_view);
-		set_page_item_sensitivity(ps_view);
-	}
-}
-
-static void
-get_status(GgvPostscriptView *ps_view, BonoboArg *arg)
+get_status(GgvPostScriptView *ps_view, BonoboArg *arg)
 {
 	GtkGS *gs = GTK_GS(ps_view->priv->gs);
 	gchar *status;
@@ -152,7 +126,7 @@ get_status(GgvPostscriptView *ps_view, BonoboArg *arg)
 }
 
 static void
-get_title(GgvPostscriptView *ps_view, BonoboArg *arg)
+get_title(GgvPostScriptView *ps_view, BonoboArg *arg)
 {
 	GtkGS *gs = GTK_GS(ps_view->priv->gs);
 	gchar *title;
@@ -170,20 +144,118 @@ get_title(GgvPostscriptView *ps_view, BonoboArg *arg)
 }
 
 static void
-load_ps(GgvPostscriptView *ps_view, const gchar *fname)
+set_page_item_sensitivity(GgvPostScriptView *ps_view)
+{
+	GtkGS *gs = GTK_GS(ps_view->priv->gs);
+	gchar *prop_val;
+
+	prop_val = (gtk_gs_get_current_page(gs) >= gtk_gs_get_page_count(gs) - 1 ||
+				gtk_gs_get_current_page(gs) < 0)?
+		"0":"1";
+	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/NextPage",
+								 "sensitive", prop_val, NULL);
+	prop_val = (gtk_gs_get_current_page(gs) <= 0 || !gs->structured_doc)?
+		"0":"1";
+	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/PrevPage",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/FirstPage",
+								 "sensitive", prop_val, NULL);
+	prop_val = (gtk_gs_get_current_page(gs) >= gtk_gs_get_page_count(gs) - 1 ||
+				!gs->structured_doc || gtk_gs_get_current_page(gs) < 0)?
+		"0":"1";
+	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/LastPage",
+								 "sensitive", prop_val, NULL);
+
+}
+
+
+static void
+notify_page_count_change(GgvPostScriptView *ps_view)
 {
 	BonoboArg *arg;
 
-	gtk_gs_load(GTK_GS(ps_view->priv->gs), fname);
+	set_page_item_sensitivity(ps_view);
+
+	arg = bonobo_arg_new(TC_CORBA_long);
+	BONOBO_ARG_SET_LONG(arg, gtk_gs_get_page_count(GTK_GS(ps_view->priv->gs)));
+	bonobo_event_source_notify_listeners(ps_view->priv->property_bag->es,
+										 "Bonobo/Property:change:page_count",
+										 arg, NULL);
+	bonobo_arg_release(arg);
+}
+
+static void
+notify_orientation_change(GgvPostScriptView *ps_view)
+{
+	BonoboArg *arg;
+
+	arg = bonobo_arg_new(TC_GNOME_GGV_Orientation);
+	*(GNOME_GGV_Orientation *)arg->_value = gtk_gs_get_orientation(GTK_GS(ps_view->priv->gs));
+	bonobo_event_source_notify_listeners(ps_view->priv->property_bag->es,
+										 "Bonobo/Property:change:orientation",
+										 arg, NULL);
+	bonobo_arg_release(arg);
+}
+
+static void
+notify_page_change(GgvPostScriptView *ps_view)
+{
+	BonoboArg *arg;
+
+	set_page_item_sensitivity(ps_view);
+
+	arg = bonobo_arg_new(TC_CORBA_long);
+	*(CORBA_long *)arg->_value = gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
+	bonobo_event_source_notify_listeners(ps_view->priv->property_bag->es,
+										 "Bonobo/Property:change:page",
+										 arg, NULL);
+	bonobo_arg_release(arg);
+}
+
+static void
+notify_title_change(GgvPostScriptView *ps_view)
+{
+	BonoboArg *arg;
 	arg = bonobo_arg_new(TC_CORBA_string);
 	get_title(ps_view, arg);
 	bonobo_event_source_notify_listeners(ps_view->priv->property_bag->es,
 										 "Bonobo/Property:change:title",
 										 arg, NULL);
 	bonobo_arg_release(arg);
-	gtk_gs_goto_page(GTK_GS(ps_view->priv->gs), 0);
-	set_page_item_sensitivity(ps_view);
-	/* TODO: notify about changes to status & sizes */
+}
+
+void
+ggv_postscript_view_goto_page(GgvPostScriptView *ps_view, gint page)
+{
+	gint old_page = gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
+
+	gtk_gs_goto_page(GTK_GS(ps_view->priv->gs), page);
+	if(gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs)) != old_page) {
+		notify_page_change(ps_view);
+	}
+}
+
+gint
+ggv_postscript_view_get_current_page(GgvPostScriptView *ps_view)
+{
+	return gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
+}
+
+gint
+ggv_postscript_view_get_page_count(GgvPostScriptView *ps_view)
+{
+	return gtk_gs_get_page_count(GTK_GS(ps_view->priv->gs));
+}
+
+
+static void
+load_ps(GgvPostScriptView *ps_view, const gchar *fname)
+{
+	gtk_gs_load(GTK_GS(ps_view->priv->gs), fname);
+	notify_title_change(ps_view);
+	notify_page_count_change(ps_view);
+	notify_orientation_change(ps_view);
+	ggv_postscript_view_goto_page(ps_view, 0);
 }
 
 /*
@@ -196,10 +268,11 @@ load_ps_from_stream (BonoboPersistStream *ps,
 					 void *data,
 					 CORBA_Environment *ev)
 {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 	guint8 *buffer;
 	CORBA_long len_read;
 	FILE *tmpfile;
+	int fd;
 
 	g_return_if_fail (data != NULL);
 	g_return_if_fail (GGV_IS_POSTSCRIPT_VIEW (data));
@@ -209,13 +282,18 @@ load_ps_from_stream (BonoboPersistStream *ps,
 	/* copy stream to a tmp file */
 	if(ps_view->priv->tmp_name != NULL) {
 		unlink(ps_view->priv->tmp_name);
+		g_free(ps_view->priv->tmp_name);
+		ps_view->priv->tmp_name = NULL;
 	}
-	else {
-		ps_view->priv->tmp_name = g_malloc(256);
+	ps_view->priv->tmp_name = g_strconcat(g_get_tmp_dir(), "/ggvXXXXXX", NULL);
+	if((fd = mkstemp(ps_view->priv->tmp_name)) < 0) {
+		g_free(ps_view->priv->tmp_name),
+		ps_view->priv->tmp_name = NULL;
+		return;
 	}
-	tmpnam(ps_view->priv->tmp_name);
-	tmpfile = fopen(ps_view->priv->tmp_name, "w");
+	tmpfile = fdopen(fd, "w");
 	if(!tmpfile) {
+		close(fd);
 		return;
 	}
 	do {
@@ -241,7 +319,7 @@ load_ps_from_stream (BonoboPersistStream *ps,
 static void
 view_button_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 
 	g_return_if_fail(data != NULL);
 	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(data));
@@ -251,6 +329,8 @@ view_button_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 	if(event->button == 1 && !ps_view->priv->pan) {
 		gint wx = 0, wy = 0;
 			
+		gtk_widget_grab_focus(ps_view->priv->gs);
+
 		gdk_window_get_pointer(widget->window, &wx, &wy, NULL);
 			
 		gtk_gs_start_scroll(GTK_GS(ps_view->priv->gs));
@@ -272,7 +352,7 @@ view_button_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 static void
 view_button_release_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 
 	g_return_if_fail(data != NULL);
 	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(data));
@@ -290,7 +370,7 @@ view_button_release_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 static void
 view_motion_cb(GtkWidget *widget, GdkEventMotion *event, gpointer data)
 {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 
 	g_return_if_fail(data != NULL);
 	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(data));
@@ -314,7 +394,7 @@ static gint
 load_ps_from_file (BonoboPersistFile *pf, const CORBA_char *filename,
 				   CORBA_Environment *ev, void *data)
 {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 	BonoboArg *arg;
 
 	g_return_val_if_fail (data != NULL, -1);
@@ -333,7 +413,7 @@ listener_Orientation_cb (BonoboUIComponent *uic, const char *path,
 						 gpointer user_data)
 {
 	BonoboArg *arg;
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 	GNOME_GGV_Orientation orientation;
 
 	g_return_if_fail(user_data != NULL);
@@ -376,7 +456,7 @@ listener_Toggle_cb (BonoboUIComponent *uic, const char *path,
 					gpointer user_data)
 {
 	BonoboArg *arg;
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 	CORBA_boolean flag;
 	gchar *prop;
 	int i;
@@ -426,13 +506,12 @@ listener_Toggle_cb (BonoboUIComponent *uic, const char *path,
 	bonobo_arg_release(arg);
 }
 
-	
 static void
 listener_Size_cb(BonoboUIComponent *uic, const char *path,
 				  Bonobo_UIComponent_EventType type, const char *state,
 				  gpointer user_data)
 {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 	GNOME_GGV_Size size;
 	BonoboArg *arg;
 
@@ -447,7 +526,7 @@ listener_Size_cb(BonoboUIComponent *uic, const char *path,
 
 	ps_view = GGV_POSTSCRIPT_VIEW(user_data);
 
-	size = CORBA_string_dup(path);
+	size = CORBA_string_dup(path + strlen("PaperSize"));
 
 	arg = bonobo_arg_new(TC_GNOME_GGV_Size);
 	BONOBO_ARG_SET_GENERAL(arg, size, TC_GNOME_GGV_Size,
@@ -462,7 +541,7 @@ listener_Size_cb(BonoboUIComponent *uic, const char *path,
 static void
 verb_NextPage_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname)
 {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 	GtkGS *gs;
 
 	g_return_if_fail (user_data != NULL);
@@ -472,14 +551,12 @@ verb_NextPage_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname)
 	gs = GTK_GS(ps_view->priv->gs);
 
 	ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
-
-	set_page_item_sensitivity(ps_view);
 }
 
 static void
 verb_PrevPage_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname)
 {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 
 	g_return_if_fail (user_data != NULL);
 	g_return_if_fail (GGV_IS_POSTSCRIPT_VIEW (user_data));
@@ -489,13 +566,12 @@ verb_PrevPage_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname)
 	ggv_postscript_view_goto_page(ps_view,
 								  GTK_GS(ps_view->priv->gs)->current_page - 1);
 
-	set_page_item_sensitivity(ps_view);
 }
 
 static void
 verb_LastPage_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname)
 {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 	GtkGS *gs;
 
 	g_return_if_fail (user_data != NULL);
@@ -505,14 +581,12 @@ verb_LastPage_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname)
 	gs = GTK_GS(ps_view->priv->gs);
 
 	ggv_postscript_view_goto_page(ps_view, gtk_gs_get_page_count(gs) - 1);
-
-	set_page_item_sensitivity(ps_view);
 }
 
 static void
 verb_FirstPage_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname)
 {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 	GtkGS *gs;
 
 	g_return_if_fail (user_data != NULL);
@@ -522,11 +596,10 @@ verb_FirstPage_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname)
 	gs = GTK_GS(ps_view->priv->gs);
 
 	ggv_postscript_view_goto_page(ps_view, 0);
-
-	set_page_item_sensitivity(ps_view);
 }
+
 static void
-sync_orientation_items(GgvPostscriptView *ps_view)
+sync_orientation_items(GgvPostScriptView *ps_view)
 {
 	gint i;
 	gboolean val, orient_state;
@@ -544,13 +617,18 @@ sync_orientation_items(GgvPostscriptView *ps_view)
 }
 
 static void
-sync_size_items(GgvPostscriptView *ps_view)
+sync_size_items(GgvPostScriptView *ps_view)
 {
 	gint i;
 	gboolean val, size_state;
-	gchar **paths;
+	const gchar **paths;
 
+#if DYNAMIC_PAPER_MENUS
 	paths = GGV_POSTSCRIPT_VIEW_CLASS(G_OBJECT_GET_CLASS(ps_view))->priv->size_paths;
+#else
+	paths = size_paths;
+#endif /* DYNAMIC_PAPER_MENUS */
+
 	val = gtk_gs_get_override_size(GTK_GS(ps_view->priv->gs));
 	bonobo_ui_component_set_prop(ps_view->priv->uic, paths[0],
 								 "state", val?"1":"0", NULL);
@@ -564,7 +642,7 @@ sync_size_items(GgvPostscriptView *ps_view)
 }
 
 static void
-ggv_postscript_view_create_ui(GgvPostscriptView *ps_view)
+ggv_postscript_view_create_ui(GgvPostScriptView *ps_view)
 {
 	static BonoboUIVerb ggv_postscript_view_verbs[] = {
 		BONOBO_UI_VERB ("NextPage", verb_NextPage_cb),
@@ -575,7 +653,7 @@ ggv_postscript_view_create_ui(GgvPostscriptView *ps_view)
 	};
 	BonoboUINode *new_size_menus;
 	BonoboUINode *ui_root;
-	GgvPostscriptViewClass *klass = GGV_POSTSCRIPT_VIEW_CLASS(G_OBJECT_GET_CLASS(ps_view));
+	GgvPostScriptViewClass *klass = GGV_POSTSCRIPT_VIEW_CLASS(G_OBJECT_GET_CLASS(ps_view));
 	gint i;
 
 	g_return_if_fail(ps_view != NULL);
@@ -586,6 +664,7 @@ ggv_postscript_view_create_ui(GgvPostscriptView *ps_view)
 						  "ggv-postscript-view-ui.xml", 
 						  "GGV", NULL);
 
+#ifdef DYNAMIC_PAPER_MENUS
 	new_size_menus = bonobo_ui_node_copy(klass->priv->size_menus, TRUE);
 
 	if(new_size_menus) {
@@ -593,23 +672,25 @@ ggv_postscript_view_create_ui(GgvPostscriptView *ps_view)
 									 new_size_menus, NULL);
 	}
 
-	bonobo_ui_component_add_listener(ps_view->priv->uic, "OrientationPortrait",
-									 listener_Orientation_cb, ps_view);
-	bonobo_ui_component_add_listener(ps_view->priv->uic, "OrientationLandscape",
-									 listener_Orientation_cb, ps_view);
-	bonobo_ui_component_add_listener(ps_view->priv->uic, "OrientationUpsideDown",
-									 listener_Orientation_cb, ps_view);
-	bonobo_ui_component_add_listener(ps_view->priv->uic, "OrientationSeascape",
-									 listener_Orientation_cb, ps_view);
-	bonobo_ui_component_add_listener(ps_view->priv->uic, "OverrideOrientation",
-									 listener_Toggle_cb, ps_view);
-	for(i = 1; klass->priv->size_paths[i] != NULL; i++) {
+	for(i = 1; klass->priv->size_paths[i]; i++) 
 		bonobo_ui_component_add_listener(ps_view->priv->uic,
 										 klass->priv->size_paths[i] + strlen("/commands/"),
 										 listener_Size_cb, ps_view);
-	}
+#else
+	for(i = 1; size_paths[i]; i++) 
+		bonobo_ui_component_add_listener(ps_view->priv->uic,
+										 size_paths[i] + strlen("/commands/"),
+										 listener_Size_cb, ps_view);
+#endif /* DYNAMIC_PAPER_MENUS */
+
+	for(i = 1; orientation_paths[i]; i++) 
+		bonobo_ui_component_add_listener(ps_view->priv->uic,
+										 orientation_paths[i] + strlen("/commands/"),
+										 listener_Orientation_cb, ps_view);
 	bonobo_ui_component_add_listener(ps_view->priv->uic, "OverrideSize",
 									 listener_Toggle_cb, ps_view);
+	bonobo_ui_component_add_listener(ps_view->priv->uic, "OverrideOrientation",
+									 listener_Toggle_cb, ps_view);
 
 	bonobo_ui_component_add_verb_list_with_data(ps_view->priv->uic,
 												ggv_postscript_view_verbs,
@@ -627,7 +708,7 @@ ggv_postscript_view_get_prop(BonoboPropertyBag *bag,
 							 CORBA_Environment *ev,
 							 gpointer           user_data)
 {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 	gchar *size;
 
 	g_return_if_fail(user_data != NULL);
@@ -817,8 +898,8 @@ ggv_postscript_view_set_prop(BonoboPropertyBag *bag,
 							 CORBA_Environment *ev,
 							 gpointer           user_data)
 {
-	GgvPostscriptView *ps_view;
-	GgvPostscriptViewClass *klass;
+	GgvPostScriptView *ps_view;
+	GgvPostScriptViewClass *klass;
 
 	g_return_if_fail(user_data != NULL);
 	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(user_data));
@@ -858,7 +939,7 @@ ggv_postscript_view_set_prop(BonoboPropertyBag *bag,
 		}
 
 		gtk_gs_set_default_orientation(GTK_GS(ps_view->priv->gs), orient);
-
+		notify_orientation_change(ps_view);
 		/*
 		sync_orientation_items(ps_view);
 		*/
@@ -868,6 +949,7 @@ ggv_postscript_view_set_prop(BonoboPropertyBag *bag,
 	case PROP_OVERRIDE_ORIENTATION: {
 		 gtk_gs_set_override_orientation(GTK_GS(ps_view->priv->gs),
 										 *(CORBA_boolean *)arg->_value);
+		 notify_orientation_change(ps_view);
 		 sync_orientation_items(ps_view);
 		 break;
 	}
@@ -918,7 +1000,7 @@ ggv_postscript_view_get_object(BonoboItemContainer *item_container,
 							   CORBA_char          *item_name,
 							   CORBA_boolean       only_if_exists,
 							   CORBA_Environment   *ev,
-							   GgvPostscriptView   *ps_view)
+							   GgvPostScriptView   *ps_view)
 {
 	Bonobo_Unknown corba_object;
 	BonoboObject *object = NULL;
@@ -965,7 +1047,7 @@ ggv_postscript_view_get_object(BonoboItemContainer *item_container,
 }
 
 BonoboObject *
-ggv_postscript_view_add_interfaces (GgvPostscriptView *ps_view,
+ggv_postscript_view_add_interfaces (GgvPostScriptView *ps_view,
 									BonoboObject *to_aggregate)
 {
 	BonoboPersistFile   *file;
@@ -988,7 +1070,7 @@ ggv_postscript_view_add_interfaces (GgvPostscriptView *ps_view,
 
 	/* Interface Bonobo::PersistFile */
 	file = bonobo_persist_file_new (load_ps_from_file, NULL,
-									"OAFIID:GNOME_GGV_PostscriptView", ps_view);
+									"OAFIID:GNOME_GGV_PostScriptView", ps_view);
 	if (!file) {
 		bonobo_object_unref (BONOBO_OBJECT (to_aggregate));
 		return NULL;
@@ -1012,7 +1094,7 @@ ggv_postscript_view_add_interfaces (GgvPostscriptView *ps_view,
 }
 
 BonoboPropertyBag *
-ggv_postscript_view_get_property_bag(GgvPostscriptView *ps_view)
+ggv_postscript_view_get_property_bag(GgvPostScriptView *ps_view)
 {
 	g_return_val_if_fail(ps_view != NULL, NULL);
 	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view), NULL);
@@ -1023,7 +1105,7 @@ ggv_postscript_view_get_property_bag(GgvPostscriptView *ps_view)
 }
 
 BonoboPropertyControl *
-ggv_postscript_view_get_property_control(GgvPostscriptView *ps_view)
+ggv_postscript_view_get_property_control(GgvPostScriptView *ps_view)
 {
 	g_return_val_if_fail(ps_view != NULL, NULL);
 	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view), NULL);
@@ -1034,7 +1116,7 @@ ggv_postscript_view_get_property_control(GgvPostscriptView *ps_view)
 }
 
 void
-ggv_postscript_view_set_ui_container(GgvPostscriptView *ps_view,
+ggv_postscript_view_set_ui_container(GgvPostScriptView *ps_view,
 				 Bonobo_UIContainer ui_container)
 {
 	g_return_if_fail(ps_view != NULL);
@@ -1047,7 +1129,7 @@ ggv_postscript_view_set_ui_container(GgvPostscriptView *ps_view,
 }
 
 void
-ggv_postscript_view_unset_ui_container(GgvPostscriptView *ps_view)
+ggv_postscript_view_unset_ui_container(GgvPostScriptView *ps_view)
 {
 	g_return_if_fail(ps_view != NULL);
 	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view));
@@ -1056,7 +1138,7 @@ ggv_postscript_view_unset_ui_container(GgvPostscriptView *ps_view)
 }
 
 GtkWidget *
-ggv_postscript_view_get_widget(GgvPostscriptView *ps_view)
+ggv_postscript_view_get_widget(GgvPostScriptView *ps_view)
 {
 	g_return_val_if_fail(ps_view != NULL, NULL);
 	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view), NULL);
@@ -1067,7 +1149,7 @@ ggv_postscript_view_get_widget(GgvPostscriptView *ps_view)
 }
 
 float
-ggv_postscript_view_get_zoom_factor(GgvPostscriptView *ps_view)
+ggv_postscript_view_get_zoom_factor(GgvPostScriptView *ps_view)
 {
 	g_return_val_if_fail(ps_view != NULL, 0.0);
 	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view), 0.0);
@@ -1076,7 +1158,7 @@ ggv_postscript_view_get_zoom_factor(GgvPostscriptView *ps_view)
 }
 
 void
-ggv_postscript_view_set_zoom_factor(GgvPostscriptView *ps_view,
+ggv_postscript_view_set_zoom_factor(GgvPostScriptView *ps_view,
 									float zoom_factor)
 {
 	g_return_if_fail(ps_view != NULL);
@@ -1087,7 +1169,7 @@ ggv_postscript_view_set_zoom_factor(GgvPostscriptView *ps_view,
 }
 
 void
-ggv_postscript_view_set_zoom(GgvPostscriptView *ps_view,
+ggv_postscript_view_set_zoom(GgvPostScriptView *ps_view,
 							 double zoomx, double zoomy)
 {
 	g_return_if_fail(zoomx > 0.0);
@@ -1099,18 +1181,18 @@ ggv_postscript_view_set_zoom(GgvPostscriptView *ps_view,
 }
 
 gfloat
-ggv_postscript_view_zoom_to_fit(GgvPostscriptView *ps_view,
+ggv_postscript_view_zoom_to_fit(GgvPostScriptView *ps_view,
 								gboolean fit_width)
 {
 	g_return_if_fail(ps_view != NULL);
 	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view));
 
 	/* we don't know how not to keep the aspect ratio */
-	return  gtk_gs_zoom_to_fit(GTK_GS(ps_view->priv->gs), fit_width);
+	return gtk_gs_zoom_to_fit(GTK_GS(ps_view->priv->gs), fit_width);
 }
 
 void
-ggv_postscript_view_set_default_orientation(GgvPostscriptView *ps_view,
+ggv_postscript_view_set_default_orientation(GgvPostScriptView *ps_view,
 											GNOME_GGV_Orientation orientation)
 {
 	BonoboArg *arg;
@@ -1127,7 +1209,7 @@ ggv_postscript_view_set_default_orientation(GgvPostscriptView *ps_view,
 }
 
 GNOME_GGV_Orientation
-ggv_postscript_view_get_default_orientation(GgvPostscriptView *ps_view)
+ggv_postscript_view_get_default_orientation(GgvPostScriptView *ps_view)
 {
 	BonoboArg *arg;
 
@@ -1146,7 +1228,7 @@ ggv_postscript_view_get_default_orientation(GgvPostscriptView *ps_view)
 static void
 ggv_postscript_view_destroy(BonoboObject *object)
 {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 
 	g_return_if_fail(object != NULL);
 	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(object));
@@ -1176,7 +1258,7 @@ ggv_postscript_view_destroy(BonoboObject *object)
 static void
 ggv_postscript_view_finalize(GObject *object)
 {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 
 	g_return_if_fail(object != NULL);
 	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(object));
@@ -1188,8 +1270,49 @@ ggv_postscript_view_finalize(GObject *object)
 	G_OBJECT_CLASS(ggv_postscript_view_parent_class)->finalize(object);
 }
 
+static CORBA_string
+impl_GNOME_GGV_PostScriptView_getDocument(PortableServer_Servant servant,
+										  CORBA_Environment *ev)
+{
+	GgvPostScriptView *ps_view;
+	gchar *doc;
+	CORBA_string retval;
+
+	ps_view = GGV_POSTSCRIPT_VIEW(bonobo_object_from_servant(servant));
+	doc = gtk_gs_get_postscript(GTK_GS(ps_view->priv->gs), NULL);
+	retval = CORBA_string_dup(doc);
+	g_free(doc);
+	return retval;
+}
+
+static CORBA_string
+impl_GNOME_GGV_PostScriptView_getPages(PortableServer_Servant servant,
+									   GNOME_GGV_PageList *pages,
+									   CORBA_Environment *ev)
+{
+	gint *page_mask;
+	GgvPostScriptView *ps_view;
+	gchar *doc;
+	CORBA_string retval;
+	GtkGS *gs;
+	int i, num;
+
+	ps_view = GGV_POSTSCRIPT_VIEW(bonobo_object_from_servant(servant));
+	gs = GTK_GS(ps_view->priv->gs);
+	num = gtk_gs_get_page_count(gs);
+	page_mask = g_new0(gint, num);
+	for(i = 0; i < pages->_length; i++) {
+		if(pages->_buffer[i] < num)
+			page_mask[pages->_buffer[i]] = TRUE;
+	}
+	doc = gtk_gs_get_postscript(gs, page_mask);
+	retval = CORBA_string_dup(doc);
+	g_free(doc);
+	return retval;
+}
+
 static void
-ggv_postscript_view_class_init(GgvPostscriptViewClass *klass)
+ggv_postscript_view_class_init(GgvPostScriptViewClass *klass)
 {
 	GObjectClass *object_class = (GObjectClass *)klass;
 	BonoboObjectClass *bonobo_object_class = (BonoboObjectClass *)klass;
@@ -1198,15 +1321,16 @@ ggv_postscript_view_class_init(GgvPostscriptViewClass *klass)
 	BonoboUINode *item;
 	gchar *enc_str;
 	GtkGSPaperSize *papersizes;
+	POA_GNOME_GGV_PostScriptView__epv *epv;
 
-	klass->priv = g_new0(GgvPostscriptViewClassPrivate, 1);
-
-#if 0
-	POA_GNOME_Ggv_PostscriptView__epv *epv;
-#endif
+	klass->priv = g_new0(GgvPostScriptViewClassPrivate, 1);
 
 	ggv_postscript_view_parent_class = g_type_class_peek_parent(klass);
 
+#ifdef DYNAMIC_PAPER_MENUS
+	/* FIXME: can't get these dynamically created menus to merge
+	   properly. for the time being we have made them static in the
+	   UI XML files */
 	/* init size menu template */
 	placeholder = bonobo_ui_node_new("placeholder");
 	bonobo_ui_node_set_attr(placeholder, "name", "SizeList");
@@ -1228,25 +1352,25 @@ ggv_postscript_view_class_init(GgvPostscriptViewClass *klass)
 	klass->priv->size_paths[0] = "/commands/OverrideSize";
 	klass->priv->size_paths[n + 1] = NULL;
 	klass->priv->size_menus = placeholder;
+#endif /* DYNAMIC_PAPER_MENUS  */
 
 	bonobo_object_class->destroy = ggv_postscript_view_destroy;
 	object_class->finalize = ggv_postscript_view_finalize;
 	
-#if 0
 	epv = &klass->epv;
 
-	epv->getImage = impl_GNOME_Ggv_PostscriptView_getImage;
-#endif
+	epv->getDocument = impl_GNOME_GGV_PostScriptView_getDocument;
+	epv->getPages = impl_GNOME_GGV_PostScriptView_getPages;
 }
 
 static void
-ggv_postscript_view_init(GgvPostscriptView *ps_view)
+ggv_postscript_view_init(GgvPostScriptView *ps_view)
 {
-	ps_view->priv = g_new0(GgvPostscriptViewPrivate, 1);
+	ps_view->priv = g_new0(GgvPostScriptViewPrivate, 1);
 }
 
-BONOBO_TYPE_FUNC_FULL(GgvPostscriptView, 
-					  GNOME_GGV_PostscriptView,
+BONOBO_TYPE_FUNC_FULL(GgvPostScriptView, 
+					  GNOME_GGV_PostScriptView,
 					  BONOBO_TYPE_OBJECT,
 					  ggv_postscript_view);
 
@@ -1271,7 +1395,7 @@ static BonoboControl *
 property_control_get_cb(BonoboPropertyControl *property_control,
 						int page_number, void *closure)
 {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 	GtkWidget *container;
 	BonoboControl *control;
 	BonoboPropertyBag *property_bag;
@@ -1306,10 +1430,24 @@ property_control_get_cb(BonoboPropertyControl *property_control,
 #endif
 }
 
+#ifdef BONOBO_DEBUG
+static void
+interpreter_message_cb(GtkGS *gs, gchar *msg, gpointer data)
+{
+	g_warning(msg);
+}
+#endif /* BONOBO_DEBUG */
 
+static void
+interpreter_error_cb(GtkGS *gs, gint status, gpointer data)
+{
+	notify_title_change(GGV_POSTSCRIPT_VIEW(data));
+	notify_page_change(GGV_POSTSCRIPT_VIEW(data));
+	notify_page_count_change(GGV_POSTSCRIPT_VIEW(data));
+}
 
-GgvPostscriptView *
-ggv_postscript_view_construct(GgvPostscriptView *ps_view,
+GgvPostScriptView *
+ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 							  GtkGS *gs, gboolean zoom_fit)
 {
 	g_return_val_if_fail(ps_view != NULL, NULL);
@@ -1323,15 +1461,23 @@ ggv_postscript_view_construct(GgvPostscriptView *ps_view,
 		gconf_init(0, NULL, NULL);
 	
 	ps_view->priv->gs = GTK_WIDGET(gs);
+	g_object_set(gs, "can-focus", TRUE, NULL);
 	gtk_widget_set_events(GTK_WIDGET(gs), 
 						  GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
 						  GDK_POINTER_MOTION_MASK | GDK_KEY_PRESS_MASK);
-	gtk_signal_connect(GTK_OBJECT(gs), "button_press_event",
-					   GTK_SIGNAL_FUNC(view_button_press_cb), ps_view);
-	gtk_signal_connect(GTK_OBJECT(gs), "button_release_event",
-					   GTK_SIGNAL_FUNC(view_button_release_cb), ps_view);
-	gtk_signal_connect(GTK_OBJECT(gs), "motion_notify_event",
-					   GTK_SIGNAL_FUNC(view_motion_cb), ps_view);
+	g_signal_connect(G_OBJECT(gs), "button_press_event",
+					 G_CALLBACK(view_button_press_cb), ps_view);
+	g_signal_connect(G_OBJECT(gs), "button_release_event",
+					 G_CALLBACK(view_button_release_cb), ps_view);
+	g_signal_connect(G_OBJECT(gs), "motion_notify_event",
+					 G_CALLBACK(view_motion_cb), ps_view);
+#ifdef BONOBO_DEBUG
+	g_signal_connect(G_OBJECT(gs), "interpreter_message",
+					 G_CALLBACK(interpreter_message_cb), NULL);
+#endif /* BONOBO_DEBUG */
+	g_signal_connect(G_OBJECT(gs), "interpreter_error",
+					 G_CALLBACK(interpreter_error_cb), ps_view);
+
 	gtk_widget_show(GTK_WIDGET(gs));
 	ps_view->priv->zoom_fit = zoom_fit;
 
@@ -1407,17 +1553,17 @@ ggv_postscript_view_construct(GgvPostscriptView *ps_view,
 		(property_control_get_cb, 1, ps_view);
 
 	/* UI Component */
-	ps_view->priv->uic = bonobo_ui_component_new("GgvPostscriptView");
+	ps_view->priv->uic = bonobo_ui_component_new("GgvPostScriptView");
 
 	ggv_postscript_view_add_interfaces (ps_view, BONOBO_OBJECT (ps_view));
 
 	return ps_view;
 }
 
-GgvPostscriptView *
+GgvPostScriptView *
 ggv_postscript_view_new(GtkGS *gs, gboolean zoom_fit)
 {
-	GgvPostscriptView *ps_view;
+	GgvPostScriptView *ps_view;
 	
 	g_return_val_if_fail(gs != NULL, NULL);
 	g_return_val_if_fail(GTK_IS_GS(gs), NULL);
@@ -1428,13 +1574,13 @@ ggv_postscript_view_new(GtkGS *gs, gboolean zoom_fit)
 }
 
 GtkAdjustment *
-ggv_postscript_view_get_hadj(GgvPostscriptView *ps_view)
+ggv_postscript_view_get_hadj(GgvPostScriptView *ps_view)
 {
 	return GTK_GS(ps_view->priv->gs)->hadj;
 }
 
 GtkAdjustment *
-ggv_postscript_view_get_vadj(GgvPostscriptView *ps_view)
+ggv_postscript_view_get_vadj(GgvPostScriptView *ps_view)
 {
 	return GTK_GS(ps_view->priv->gs)->vadj;
 }
