@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
- * main.c: the ggv shell
+ * ggv-window.c: the ggv shell
  *
  * Copyright (C) 2002 the Free Software Foundation
  *
@@ -35,6 +35,97 @@ enum {
         TARGET_URI_LIST,
 };
 
+static CORBA_string
+ggv_window_get_ps_from_control(GgvWindow *win, gint *active_rows)
+{
+        GNOME_GGV_PostScriptView ggv;
+        CORBA_Environment ev;
+        CORBA_string doc;
+        GNOME_GGV_PageList *page_list;
+        FILE *f;
+        CORBA_long page_count;
+        gint i;
+
+        CORBA_exception_init(&ev);
+
+        /* get reference of our PostScriptView interface */
+        ggv = Bonobo_Unknown_queryInterface(win->control,
+                                            "IDL:GNOME/GGV/PostScriptView:1.0", &ev);
+        if(BONOBO_EX(&ev) || ggv == CORBA_OBJECT_NIL) {
+                CORBA_exception_free(&ev);
+                return CORBA_OBJECT_NIL;
+        }
+
+        /* TODO: check if the doc is structured */
+
+        /* allocate space for page list */
+        page_count = bonobo_pbclient_get_long(win->pb, "page_count", NULL);
+        page_list = CORBA_sequence_CORBA_long__alloc();
+        page_list->_maximum = page_count;
+        page_list->_length = 0;
+        page_list->_buffer = CORBA_sequence_CORBA_long_allocbuf(page_count);
+        page_list->_release = TRUE;
+        /* and fill it in if necessary */
+        if(active_rows) {
+                i = 0;
+                while(active_rows[i] >= 0) {
+                        page_list->_buffer[page_list->_length++] = active_rows[i++];
+                }
+        }
+
+        if(page_list->_length == 0)
+                doc = GNOME_GGV_PostScriptView_getDocument(ggv, &ev);
+        else
+                doc = GNOME_GGV_PostScriptView_getPages(ggv, page_list, &ev);
+
+        CORBA_free(page_list);
+
+        CORBA_exception_free(&ev);
+
+        return doc;
+}
+
+static void
+ggv_window_print_postscript(GgvWindow *win, CORBA_string ps)
+{
+        gchar **argv;
+        gint print_in;
+
+        argv = g_strsplit(ggv_print_cmd, " ", 0);
+        if(!g_spawn_async_with_pipes(NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
+                                     NULL, NULL, NULL, &print_in, NULL, NULL,
+                                     NULL)) {
+                GtkWidget *dlg;
+                dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                             GTK_DIALOG_MODAL,
+                                             GTK_MESSAGE_ERROR,
+                                             GTK_BUTTONS_OK,
+                                             _("Unable to execute print command:\n%s"),
+                                             ggv_print_cmd);
+                gtk_widget_show(dlg);
+                gtk_dialog_run(GTK_DIALOG(dlg));
+                gtk_widget_destroy(dlg);
+        }
+        else {
+                write(print_in, ps, strlen(ps));
+                close(print_in);
+        }
+        g_strfreev(argv);
+}
+
+static void
+set_file_items_sensitivity(GgvWindow *win, gboolean sens)
+{
+        gchar *prop_val = sens?"1":"0";
+
+	bonobo_ui_component_set_prop(win->uic, "/commands/FileSaveMarked",
+                                     "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(win->uic, "/commands/FilePrintMarked",
+                                     "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(win->uic, "/commands/FilePrintAll",
+                                     "sensitive", prop_val, NULL);
+}
+
 static void
 control_property_changed_handler(BonoboListener    *listener,
                                  char              *event_name, 
@@ -58,16 +149,34 @@ control_property_changed_handler(BonoboListener    *listener,
                 gchar path_str[16];
                 GtkTreePath *path;
 
-                g_message("Page change: %d.", BONOBO_ARG_GET_INT(any));
+                if(win->current_page == BONOBO_ARG_GET_INT(any) &&
+                   win->current_page < 0)
+                        return;
+
+                win->current_page = BONOBO_ARG_GET_INT(any);
+                set_file_items_sensitivity(win, win->current_page != -1);
+                if(win->current_page < 0) {
+                        GtkWidget *dlg;
+                        dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                                     GTK_DIALOG_MODAL,
+                                                     GTK_MESSAGE_ERROR,
+                                                     GTK_BUTTONS_OK,
+                                                     _("Ghostscript error:\nProbably the file is not a PostScript file."));
+                        gtk_widget_show(dlg);
+                        gtk_dialog_run(GTK_DIALOG(dlg));
+                        gtk_widget_destroy(dlg);
+                        return;
+                }
 
                 sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(win->checklist));
-                win->current_page = BONOBO_ARG_GET_INT(any);
                 g_snprintf(path_str, 15, "%d", BONOBO_ARG_GET_INT(any));
                 path = gtk_tree_path_new_from_string(path_str);
                 if(!gtk_tree_selection_path_is_selected(sel, path))
                         gtk_tree_selection_select_path(sel, path);
                 gtk_tree_path_free(path);
         }
+        else if(!g_strcasecmp(event_name, "Bonobo/Property:change:page_count")) {
+        }
 }
 
 static void
@@ -77,6 +186,7 @@ get_window_size(GtkWidget *widget, gint *width, gint *height)
         *height = widget->allocation.height;
 }
 
+
 static void
 file_sel_unmap(GtkWidget *widget)
 {
@@ -99,7 +209,16 @@ file_sel_ok_clicked(GtkWidget *widget, gpointer data)
         gtk_widget_hide(win->file_sel);
         fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(win->file_sel));
         if(!ggv_window_load(win, fname)) {
-                /* TODO: report an error */
+                GtkWidget *dlg;
+                dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                             GTK_DIALOG_MODAL,
+                                             GTK_MESSAGE_ERROR,
+                                             GTK_BUTTONS_OK,
+                                             _("Unable to load file:\n%s"),
+                                             fname);
+                gtk_widget_show(dlg);
+                gtk_dialog_run(GTK_DIALOG(dlg));
+                gtk_widget_destroy(dlg);
         }
 }
 
@@ -111,6 +230,40 @@ file_sel_cancel_clicked(GtkWidget *widget, gpointer data)
         gtk_widget_hide(win->file_sel);
 }
 
+static void
+save_file_sel_ok_clicked(GtkWidget *widget, gpointer data)
+{
+        GgvWindow *win = GGV_WINDOW(data);
+        gint *active_rows;
+        const gchar *fname;
+        CORBA_string doc;
+        FILE *f;
+
+        active_rows = gtk_check_list_get_active_list(GTK_CHECK_LIST(win->checklist));
+        doc = ggv_window_get_ps_from_control(win, active_rows);
+        g_free(active_rows);
+
+        if(doc != CORBA_OBJECT_NIL) {
+                /* get file name */
+                gtk_widget_hide(win->save_file_sel);
+                fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(win->save_file_sel));
+
+                if(fname && (f = fopen(fname, "w")) != NULL) {
+                        fwrite(doc, 1, strlen(doc), f);
+                        fclose(f);
+                }
+                CORBA_free(doc);
+        }
+}
+
+static void
+save_file_sel_cancel_clicked(GtkWidget *widget, gpointer data)
+{
+        GgvWindow *win = GGV_WINDOW(data);
+
+        gtk_widget_hide(win->save_file_sel);
+}
+
 static void
 verb_FileOpen(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
@@ -144,6 +297,104 @@ verb_FileClose(BonoboUIComponent *uic, gpointer data, const char *cname)
         ggv_window_close(win);
 }
 
+static void
+verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
+{
+        GgvWindow *win = GGV_WINDOW(data);
+        gint *active_rows;
+
+        active_rows = gtk_check_list_get_active_list(GTK_CHECK_LIST(win->checklist));
+        if(!active_rows) {
+                GtkWidget *dlg;
+
+                dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                             GTK_DIALOG_MODAL,
+                                             GTK_MESSAGE_QUESTION,
+                                             GTK_BUTTONS_YES_NO,
+                                             _("No pages have been marked.\n"
+                                               "Do you want to save the whole document?"));
+                gtk_widget_show(dlg);
+                switch(gtk_dialog_run(GTK_DIALOG(dlg))) {
+                case GTK_RESPONSE_NO:
+                case GTK_RESPONSE_DELETE_EVENT:
+                        return;
+                default:
+                        break;
+                }
+                gtk_widget_destroy(dlg);
+        }
+        else
+                g_free(active_rows);
+        
+
+        if(!win->save_file_sel) {
+                win->save_file_sel = gtk_file_selection_new(_("Select a file to save pages as"));
+                gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(win->save_file_sel));
+                gtk_window_set_resizable(GTK_WINDOW(win->save_file_sel), TRUE);
+                g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(win->save_file_sel)->ok_button),
+                                 "clicked", G_CALLBACK(save_file_sel_ok_clicked), win);
+                g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(win->save_file_sel)->cancel_button),
+                                 "clicked", G_CALLBACK(save_file_sel_cancel_clicked), win);
+                g_signal_connect(G_OBJECT(win->save_file_sel), "delete_event",
+                                 G_CALLBACK(file_sel_cancel_clicked), win);
+                g_signal_connect(G_OBJECT(win->save_file_sel), "unmap",
+                                 G_CALLBACK(file_sel_unmap), NULL);
+        }
+        if(!GTK_WIDGET_VISIBLE(win->save_file_sel)) {
+                gtk_widget_set_usize(GTK_WIDGET(win->save_file_sel),
+                                     file_sel_width, file_sel_height);
+                gtk_widget_show(win->save_file_sel);
+        }
+}
+
+static void
+verb_FilePrintMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
+{
+        GgvWindow *win = GGV_WINDOW(data);
+        CORBA_string ps;
+        gint *active_rows;
+
+        active_rows = gtk_check_list_get_active_list(GTK_CHECK_LIST(win->checklist));
+        if(!active_rows) {
+                GtkWidget *dlg;
+
+                dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                             GTK_DIALOG_MODAL,
+                                             GTK_MESSAGE_QUESTION,
+                                             GTK_BUTTONS_YES_NO,
+                                             _("No pages have been marked.\n"
+                                               "Do you want to print the whole document?"));
+                gtk_widget_show(dlg);
+                switch(gtk_dialog_run(GTK_DIALOG(dlg))) {
+                case GTK_RESPONSE_NO:
+                case GTK_RESPONSE_DELETE_EVENT:
+                        return;
+                default:
+                        break;
+                }
+                gtk_widget_destroy(dlg);
+        }
+        if((ps = ggv_window_get_ps_from_control(win, active_rows)) != CORBA_OBJECT_NIL) {
+                ggv_window_print_postscript(win, ps);
+                CORBA_free(ps);
+        }
+        if(active_rows)
+                g_free(active_rows);
+}
+
+static void
+verb_FilePrintAll(BonoboUIComponent *uic, gpointer data, const char *cname)
+{
+        GgvWindow *win = GGV_WINDOW(data);
+
+        CORBA_string ps;
+
+        if((ps = ggv_window_get_ps_from_control(win, NULL)) != CORBA_OBJECT_NIL) {
+                ggv_window_print_postscript(win, ps);
+                CORBA_free(ps);
+        }
+}
+
 static void
 verb_FileNew(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
@@ -277,6 +528,9 @@ static BonoboUIVerb ggv_app_verbs[] = {
         BONOBO_UI_VERB("FileNew", verb_FileNew),
         BONOBO_UI_VERB("FileClose", verb_FileClose),
         BONOBO_UI_VERB("FileOpen", verb_FileOpen),
+        BONOBO_UI_VERB("FileSaveMarked", verb_FileSaveMarked),
+        BONOBO_UI_VERB("FilePrintMarked", verb_FilePrintMarked),
+        BONOBO_UI_VERB("FilePrintAll", verb_FilePrintAll),
         BONOBO_UI_VERB("FileExit", verb_FileExit),
         BONOBO_UI_VERB("SettingsPreferences", verb_SettingsPreferences),
         BONOBO_UI_VERB("HelpAbout", verb_HelpAbout),
@@ -632,6 +886,7 @@ ggv_window_new(void)
 
 	/* add menu and toolbar */
 	uic = bonobo_ui_component_new("ggv");
+        win->uic = uic;
 	bonobo_ui_component_set_container(uic, BONOBO_OBJREF(ui_container),
                                           NULL);
 	bonobo_ui_util_set_ui(uic, NULL, "ggv-ui.xml", "GGV", NULL);
@@ -688,6 +943,8 @@ ggv_window_new(void)
                 win->pb = pb;
         }
 
+        set_file_items_sensitivity(win, FALSE);
+
 	/* set default geometry */
         gtk_widget_set_usize(GTK_WIDGET(win),
                              ggv_default_width, ggv_default_height);
@@ -701,6 +958,7 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
 {
         CORBA_Environment ev;
         Bonobo_PersistFile pf;
+        gchar *full_path;
 
         g_return_val_if_fail(win != NULL, FALSE);
         g_return_val_if_fail(GGV_IS_WINDOW(win), FALSE);
@@ -717,8 +975,27 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
                 CORBA_exception_free(&ev);
                 return FALSE;
         }
-        Bonobo_PersistFile_load(pf, filename, &ev);
+
+        /* we need to pass full path to file to the control as we
+           do not really know where it resides.
+           NOTE: this still only works when the control is running
+           on the same computer as the GGV. I suppose this will mostly
+           be the case ;) */
+        if(*filename != '/') {
+                gchar *cwd;
+
+                cwd = g_get_current_dir();
+                full_path = g_strconcat(cwd, "/", filename, NULL);
+                g_free(cwd);
+        }
+        else
+                full_path = g_strdup(filename);
+
+        Bonobo_PersistFile_load(pf, full_path, &ev);
         bonobo_object_release_unref(pf, NULL);
+
+        g_free(full_path);
+
         if(BONOBO_EX(&ev)) {
                 CORBA_exception_free(&ev);
                 return FALSE;
