@@ -7,9 +7,6 @@
 
 #include "ggvutils.h"
 
-#include "stock/stock-zoom-in.xpm"
-#include "stock/stock-zoom-out.xpm"
-
 #include "stock/toggleall.xpm"
 #include "stock/clearall.xpm"
 #include "stock/toggleeven.xpm"
@@ -34,14 +31,14 @@ GtkGSPaperSize ggv_paper_sizes[] = {
         {NULL,		   0,    0}
 };
 
-gfloat ggv_unit_factors[] = {
+const gfloat ggv_unit_factors[] = {
         1.0,
         25.4,
         2.54,
         72.0
 };
 
-char *ggv_orientation_labels[] = {
+const gchar *ggv_orientation_labels[] = {
         N_("Portrait"),
         N_("Landscape"),
         N_("Upside Down"),
@@ -49,7 +46,10 @@ char *ggv_orientation_labels[] = {
         NULL,
 };
 
-char *ggv_unit_labels[] = {
+const gint ggv_max_orientation_labels =
+    (sizeof(ggv_orientation_labels) / sizeof(gchar *)) - 2;
+
+const gchar *ggv_unit_labels[] = {
         N_("inch"),
         N_("mm"),
         N_("cm"),
@@ -57,6 +57,52 @@ char *ggv_unit_labels[] = {
         NULL
 };
 
+const gint ggv_max_unit_labels =
+    (sizeof(ggv_unit_labels) / sizeof(gchar *)) - 2;
+
+const gfloat ggv_zoom_levels[] = {
+	1.0 / 10.0, 1.0 / 9.0, 1.0 / 8.0, 1.0 / 7.0, 1.0 / 6.0,
+	1.0 / 5.0, 1.0 / 4.0, 1.0 / 3.0, 1.0 / 2.0, 1.0, 2.0,
+	3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0
+};
+
+const gchar *ggv_zoom_level_names[] = {
+	"1:10", "1:9", "1:8", "1:7", "1:6", "1:5", "1:4", "1:3",
+	"1:2", "1:1", "2:1", "3:1", "4:1", "5:1", "6:1", "7:1",
+	"8:1", "9:1", "10:1"
+};
+
+const gint ggv_max_zoom_levels =
+    (sizeof (ggv_zoom_levels) / sizeof (gfloat)) - 1;
+
+gint
+ggv_zoom_index_from_float (gfloat zoom_level)
+{
+	int i;
+
+	for (i = 0; i <= ggv_max_zoom_levels; i++) {
+		float this, epsilon;
+
+		/* if we're close to a zoom level */
+		this = ggv_zoom_levels [i];
+		epsilon = this * 0.01;
+
+		if (zoom_level < this+epsilon)
+			return i;
+	}
+
+	return ggv_max_zoom_levels;
+}
+
+gfloat
+ggv_zoom_level_from_index (gint index)
+{
+	if (index > ggv_max_zoom_levels)
+		index = ggv_max_zoom_levels;
+
+	return ggv_zoom_levels [index];
+}
+
 GSList *
 ggv_split_string (const gchar *string, const gchar *delimiter)
 {
@@ -155,6 +201,7 @@ gchar *ggv_quote_filename (const gchar *str)
                 case '^':
                 case '$':
                 case '`':  /* expansion characters */
+                case '%':
                         *t++ = '\\';
                         *t++ = c;
                         break;
@@ -168,7 +215,7 @@ gchar *ggv_quote_filename (const gchar *str)
                         break;
                 }
         }
-                *t++ = '\0';
+        *t++ = '\0';
         return buf;
 }
 
@@ -201,7 +248,8 @@ void ggv_set_tooltip(GtkWidget* w, const gchar* tip)
 
 void ggv_stock_init (void)
 {
-	static GnomeStockPixmapEntry entries[6];
+        //FIXME
+	/*static GnomeStockPixmapEntry entries[6];
 
 	entries[0].data.type = GNOME_STOCK_PIXMAP_TYPE_DATA;
 	entries[0].data.width = 24;
@@ -239,7 +287,7 @@ void ggv_stock_init (void)
 	gnome_stock_pixmap_register (STOCK_TOGGLE_EVEN, GNOME_STOCK_PIXMAP_REGULAR, &entries[3]);
 	gnome_stock_pixmap_register (STOCK_TOGGLE_ODD, GNOME_STOCK_PIXMAP_REGULAR, &entries[4]);
 	gnome_stock_pixmap_register (STOCK_CLEAR_ALL, GNOME_STOCK_PIXMAP_REGULAR, &entries[5]);
-
+        */
 }
 
 gfloat ggv_compute_zoom(gint zoom_spec)
@@ -256,3 +304,10 @@ gint ggv_compute_spec(gfloat zoom)
         return (gint)rint(zoom);
 }
 
+void
+ggv_raise_and_focus_widget (GtkWidget *widget)
+{
+	g_assert (GTK_WIDGET_REALIZED (widget));
+	gdk_window_show (widget->window);
+	gtk_widget_grab_focus (widget);
+}
