@@ -109,10 +109,15 @@ The DONE message indicates that ghostscript has finished processing.
 
 #include "config.h"
 #include <string.h>
+#include <stdlib.h>
 #include <signal.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkprivate.h>
 #include <gdk/gdkx.h>
+#ifdef  HAVE_XINERAMA
+#  include <gdk/gdkx.h>
+#  include <X11/extensions/Xinerama.h>
+#endif /* HAVE_XINERAMA */
 #include <X11/Intrinsic.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -127,9 +132,6 @@ The DONE message indicates that ghostscript has finished processing.
 #include "ps.h"
 #include "gsdefaults.h"
 
-//#include "ggv-marshal.h"
-
-
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
@@ -142,16 +144,13 @@ The DONE message indicates that ghostscript has finished processing.
 #define GTK_GS_WATCH_INTERVAL 1000
 #define GTK_GS_WATCH_TIMEOUT  2
 
-typedef void (*InterpreterMessageSignal)(GtkObject *object, gchar *msg,
-                                         gpointer data);
-
-enum { INTERPRETER_MESSAGE, DOCUMENT_CHANGED, LAST_SIGNAL };
+enum { INTERPRETER_MESSAGE, INTERPRETER_ERROR, DOCUMENT_CHANGED, LAST_SIGNAL };
 
 static gboolean broken_pipe = FALSE;
 
 static void catchPipe(int i)
 {
-        g_print ("in catch Pipe!!!\n");
+        g_message("in catch Pipe!!!\n");
         broken_pipe = True;
 }
 
@@ -222,13 +221,45 @@ ggv_marshaller_VOID__POINTER(GClosure     *closure,
             data2);
 }
 
+static void
+ggv_marshaller_VOID__INT(GClosure     *closure,
+				  GValue       *return_value,
+                                  guint         n_param_values,
+                                  const GValue *param_values,
+                                  gpointer      invocation_hint,
+                                  gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__INT) (gpointer     data1,
+                                          gint         arg_1,
+                                          gpointer     data2);
+  register GMarshalFunc_VOID__INT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure)) {
+          data1 = closure->data;
+          data2 = g_value_peek_pointer (param_values + 0);
+  }
+  else {
+          data1 = g_value_peek_pointer (param_values + 0);
+          data2 = closure->data;
+  }
+  callback = (GMarshalFunc_VOID__INT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_value_get_int (param_values + 1),
+            data2);
+}
+
 static void
 gtk_gs_init (GtkGS *gs)
 {
         gs->bpixmap     = NULL;
         gs->use_bpixmap = TRUE;
 
-        gs->current_page = -1;
+        gs->current_page = -2;
         gs->disable_start = FALSE;
         gs->interpreter_pid = -1;
 
@@ -274,7 +305,6 @@ gtk_gs_init (GtkGS *gs)
         gs->top_margin = 0;
         gs->right_margin = 0;
         gs->bottom_margin = 0;
-        gs->pages_marked = NULL;
 
         /* Set user defined defaults */
         gs->override_orientation = gtk_gs_defaults_get_override_orientation();
@@ -316,6 +346,16 @@ gtk_gs_class_init (GtkGSClass *klass)
                        ggv_marshaller_VOID__POINTER,
                        G_TYPE_NONE,
                        1, G_TYPE_POINTER);
+        gtk_gs_signals[INTERPRETER_ERROR] = g_signal_new(
+                       "interpreter_error",
+                       G_TYPE_FROM_CLASS(object_class),
+                       G_SIGNAL_RUN_LAST,
+                       GTK_SIGNAL_OFFSET(GtkGSClass, interpreter_error),
+                       NULL,
+                       NULL,
+                       ggv_marshaller_VOID__INT,
+                       G_TYPE_NONE,
+                       1, G_TYPE_INT);
 
         object_class->destroy = gtk_gs_destroy;
 
@@ -326,7 +366,7 @@ gtk_gs_class_init (GtkGSClass *klass)
         /* Create atoms */
         klass->gs_atom        = gdk_atom_intern ("GHOSTVIEW", FALSE);
         klass->gs_colors_atom = gdk_atom_intern ("GHOSTVIEW_COLORS", FALSE);
-        klass->next_atom      = gdk_atom_intern ("NEXT", TRUE);
+        klass->next_atom      = gdk_atom_intern ("NEXT", FALSE);
         klass->page_atom      = gdk_atom_intern ("PAGE", FALSE);
         klass->done_atom      = gdk_atom_intern ("DONE", FALSE);
         klass->string_atom    = gdk_atom_intern ("STRING", FALSE);
@@ -370,13 +410,6 @@ gtk_gs_defaults_changed(GConfClient* client, guint cnxn_id,
                                                                       NULL));
 }
 
-/* free message as it was allocated in output() */
-static void
-gtk_gs_interpreter_message(GtkGS *gs, gchar *msg, gpointer user_data)
-{
-        g_free(msg);
-}
-
 /* Clean all memory and temporal files */
 static void 
 gtk_gs_cleanup (GtkGS *gs)
@@ -408,19 +441,31 @@ gtk_gs_cleanup (GtkGS *gs)
                 g_free(gs->gs_filename_unc);
                 gs->gs_filename_unc = NULL;
         }
-        if (gs->pages_marked) {
-                g_free(gs->pages_marked);
-                gs->pages_marked = NULL;
-	}
         if(gs->timer_tag) {
  		gtk_timeout_remove (gs->timer_tag);
  		gs->timer_tag = 0;
  	}
+        if(!gdk_window_is_visible(gs->pstarget))
+                gdk_window_hide(gs->pstarget);
+        gs->current_page = -1;
         gs->loaded = FALSE;
         gs->llx = 0;
         gs->lly = 0;
         gs->urx = 0;
         gs->ury = 0;
+        set_up_page(gs);
+}
+
+/* free message as it was allocated in output() */
+static void
+gtk_gs_interpreter_message(GtkGS *gs, gchar *msg, gpointer user_data)
+{
+        if(strstr(msg, "Error:")) {
+                gtk_gs_cleanup(gs);
+                g_signal_emit_by_name(G_OBJECT(gs), "interpreter_error",
+                                      1, NULL);
+        }
+        g_free(msg);
 }
 
 static void
@@ -596,8 +641,6 @@ gtk_gs_size_request (GtkWidget      *widget,
         GtkGS *gs = GTK_GS (widget);
 
 	compute_size(gs);
-	if(gs->width == 0 || gs->height == 0)
-                return;
 	requisition->width = gs->width;
 	requisition->height = gs->height;
 }
@@ -627,10 +670,11 @@ gtk_gs_size_allocate (GtkWidget     *widget,
                 event.height = allocation->height;
                 gtk_widget_event (widget, (GdkEvent*) &event);
         }
+
         /* 
          * update the adjustment if necessary (ie. a resize);
          */
-        if ((gs->width > 0) && (gs->height >0) && GTK_WIDGET_REALIZED(gs)) {
+        if (GTK_WIDGET_REALIZED(gs)) {
 		gtk_gs_munge_adjustments (gs);
         }
 
@@ -811,26 +855,32 @@ set_up_page (GtkGS *gs)
         
         orientation = gtk_gs_get_orientation(gs);
         
-        if ((gs->use_bpixmap) && compute_size (gs)) {
-                gdk_flush(); 
+        if (compute_size (gs)) {
+                gdk_flush();
 
-                if (gs->bpixmap) {
-                        gdk_drawable_unref(gs->bpixmap);
-                        gs->bpixmap = NULL;
-                }
-                gs->bpixmap = gdk_pixmap_new(gs->pstarget, gs->width, gs->height, -1);
- 
                 /* clear new pixmap (set to white) */
                 fill = gdk_gc_new (gs->pstarget);
                 colormap = gtk_widget_get_colormap (GTK_WIDGET(gs));
                 gdk_color_alloc (colormap, &white);
                 gdk_gc_set_foreground (fill, &white);
-                gdk_draw_rectangle (gs->bpixmap, fill, TRUE,
-                                    0, 0, gs->width, gs->height);
+ 
+                if(gs->use_bpixmap) {
+                        if (gs->bpixmap) {
+                                gdk_drawable_unref(gs->bpixmap);
+                                gs->bpixmap = NULL;
+                        }
+                        gs->bpixmap = gdk_pixmap_new(gs->pstarget, gs->width, gs->height, -1);
+ 
+                        gdk_draw_rectangle (gs->bpixmap, fill, TRUE,
+                                            0, 0, gs->width, gs->height);
+                        gdk_window_set_back_pixmap(gs->pstarget, gs->bpixmap, FALSE);
+                }
+                else {
+                        gdk_draw_rectangle (gs->pstarget, fill, TRUE,
+                                            0, 0, gs->width, gs->height);
+                }
                 gdk_gc_unref(fill);
 
-                gdk_window_set_back_pixmap(gs->pstarget, gs->bpixmap, FALSE);
-
                 gdk_window_resize (gs->pstarget, gs->width, gs->height); 
 
                 gdk_flush();
@@ -845,17 +895,19 @@ set_up_page (GtkGS *gs)
 #endif
         pprivate = (GdkPixmap *) gs->bpixmap;
 
-        snprintf(buf, 1024, "%ld %d %d %d %d %d %f %f %d %d %d %d",
-                 gdk_x11_drawable_get_xid(pprivate), 
-                 orientation * 90,
-                 gs->llx, gs->lly,
-                 gs->urx, gs->ury,
-                 gs->xdpi * gs->zoom_factor,
-                 gs->ydpi * gs->zoom_factor,
-                 gs->left_margin,
-                 gs->bottom_margin,
-                 gs->right_margin,
-                 gs->top_margin);
+        g_snprintf(buf, 1024, "%ld %d %d %d %d %d %f %f %d %d %d %d",
+                   pprivate?gdk_x11_drawable_get_xid(pprivate):0L,
+                   orientation * 90,
+                   gs->llx,
+                   gs->lly,
+                   gs->urx,
+                   gs->ury,
+                   gs->xdpi * gs->zoom_factor,
+                   gs->ydpi * gs->zoom_factor,
+                   gs->left_margin,
+                   gs->bottom_margin,
+                   gs->right_margin,
+                   gs->top_margin);
 
 #ifdef HAVE_SETLOCALE
         setlocale (LC_NUMERIC, savelocale);
@@ -913,6 +965,9 @@ output (gpointer data, gint source, GdkInputCondition condition)
                         interpreter_failed(gs);
                         return;
                 }
+                if(gs->interpreter_err == -1) {
+                        stop_interpreter(gs);
+                }
         }
         else if (source == gs->interpreter_err) {
                 bytes = read(gs->interpreter_err, buf, MAX_BUFSIZE);
@@ -926,6 +981,9 @@ output (gpointer data, gint source, GdkInputCondition condition)
                         interpreter_failed(gs);
                         return;
                 }
+                if(gs->interpreter_output == -1) {
+                        stop_interpreter(gs);
+                }
         }
         if (bytes > 0) {
                 buf[bytes] = '\0';
@@ -1126,11 +1184,10 @@ start_interpreter (GtkGS *gs)
                         }
                 }
 
-                 gv_env = g_strdup_printf ("GHOSTVIEW=%ld",
+                gv_env = g_strdup_printf ("GHOSTVIEW=%ld",
                                           gdk_x11_drawable_get_xid(gs->pstarget) );
                 putenv (gv_env);
 
-
                 /* change to directory where the input file is. This helps
                  * with postscript-files which include other files using
                  * a relative path */
@@ -1176,8 +1233,14 @@ static void
 stop_interpreter (GtkGS *gs)
 {
         if (gs->interpreter_pid >= 0) {
+                int status = 0;
                 kill(gs->interpreter_pid, SIGTERM);
-                wait(0);
+                wait(&status);
+                if(status == 1) {
+                        gtk_gs_cleanup(gs);
+                        g_signal_emit_by_name(G_OBJECT(gs), "interpreter_error",
+                                              status);
+                }
 		gs->interpreter_pid = -1;
 	}
 
@@ -1232,6 +1295,7 @@ check_filecompressed (GtkGS *gs)
 	gchar buf[1024];
 	gchar *filename, *filename_unc, *filename_err, *cmdline;
         const gchar *cmd;
+        int fd;
 
 	cmd = NULL;
 
@@ -1253,8 +1317,19 @@ check_filecompressed (GtkGS *gs)
 
 	/* do the decompression */
 	filename = ggv_quote_filename (gs->gs_filename);
-	filename_unc = tempnam (g_get_tmp_dir(), "unc");
-	filename_err = tempnam (g_get_tmp_dir(), "err");
+	filename_unc = g_strconcat (g_get_tmp_dir(), "/ggvXXXXXX", NULL);
+        if((fd = mkstemp(filename_unc)) < 0) {
+                g_free(filename);
+                return NULL;
+        }
+        close(fd);
+	filename_err = g_strconcat (g_get_tmp_dir(), "/ggvXXXXXX", NULL);
+        if((fd = mkstemp(filename_err)) < 0) {
+                g_free(filename_unc);
+                g_free(filename);
+                return NULL;
+        }
+        close(fd);
 	cmdline = g_strdup_printf ("%s %s >%s 2>%s", cmd,
 				   filename, filename_unc, filename_err);
 	if ((system (cmdline) == 0)
@@ -1298,6 +1373,7 @@ check_pdf (GtkGS *gs)
 {
 	FILE *file;
 	gchar buf[1024], *filename;
+        int fd;
 
 	/* use uncompressed file as input if necessary */
 	filename = (gs->gs_filename_unc ? gs->gs_filename_unc
@@ -1308,9 +1384,18 @@ check_pdf (GtkGS *gs)
 	    && (strncmp (buf, "%PDF-", 5) == 0)) {
 		/* we found a PDF file */
 		gchar *fname, *filename_dsc, *filename_err, *cmd, *cmdline;
+		filename_dsc = g_strconcat (g_get_tmp_dir(), "/ggvXXXXXX", NULL);
+                if((fd = mkstemp(filename_dsc)) < 0) {
+                        return NULL;
+                }
+                close(fd);
+		filename_err = g_strconcat (g_get_tmp_dir(), "/ggvXXXXXX", NULL);
+                if((fd = mkstemp(filename_err)) < 0) {
+                        g_free(filename_dsc);
+                        return NULL;
+                }
+                close(fd);
 		fname = ggv_quote_filename (filename);
-		filename_dsc = tempnam (g_get_tmp_dir(), "dsc");
-		filename_err = tempnam (g_get_tmp_dir(), "err");
 		cmd = g_strdup_printf (gtk_gs_defaults_get_scan_pdf_cmd(),
                                        fname, filename_dsc);
 		g_free (fname);
@@ -1327,7 +1412,7 @@ check_pdf (GtkGS *gs)
 			/* report error */
 			g_snprintf (buf, 1024,
 				    _("Error while scanning pdf file %s:\n"),
-				   filename);
+                                    filename);
 			gtk_gs_emit_error_msg (gs, buf);
 
 			if (ggv_file_length (filename_err) > 0) {
@@ -1353,13 +1438,41 @@ check_pdf (GtkGS *gs)
 static float
 compute_xdpi(void)
 {
+#ifndef HAVE_XINERAMA
         return 25.4 * gdk_screen_width()/gdk_screen_width_mm();
+#else
+        Display *dpy;
+        dpy = (Display *) GDK_DISPLAY ();
+        if (XineramaIsActive (dpy)) {
+                int num_heads;
+                XineramaScreenInfo *head_info;
+                head_info = (XineramaScreenInfo *) XineramaQueryScreens (dpy, &num_heads);
+                /* fake it with dimensions of the first head for now */
+                return 25.4 * head_info[0].width/gdk_screen_width_mm();
+        } else {
+                return 25.4 * gdk_screen_width()/gdk_screen_width_mm();
+        }
+#endif /* HAVE_XINERAMA */
 }
-
+ 
 static float
 compute_ydpi(void)
 {
+#ifndef HAVE_XINERAMA
         return 25.4 * gdk_screen_height()/gdk_screen_height_mm();
+#else
+        Display *dpy;
+        dpy = (Display *) GDK_DISPLAY ();
+        if (XineramaIsActive (dpy)) {
+                int num_heads;
+                XineramaScreenInfo *head_info;
+                head_info = (XineramaScreenInfo *) XineramaQueryScreens (dpy, &num_heads);
+                /* fake it with dimensions of the first head for now */
+                return 25.4 * head_info[0].height/gdk_screen_height_mm();
+        } else {
+                return 25.4 * gdk_screen_height()/gdk_screen_height_mm();
+        }
+#endif /* HAVE_XINERAMA */
 }
 
 /* Compute new size of window, sets xdpi and ydpi if necessary.
@@ -1372,8 +1485,8 @@ compute_size(GtkGS *gs)
         gboolean change = FALSE;
         gint orientation;
 
-	/* width and height can be changed, calculate window size according */
-	/* to xpdi and ydpi */
+        /* width and height can be changed, calculate window size according */
+        /* to xpdi and ydpi */
         orientation = gtk_gs_get_orientation(gs);
 
         switch(orientation) {
@@ -1423,7 +1536,8 @@ gtk_gs_enable_interpreter (GtkGS *gs)
 	gs->disable_start = FALSE;
 	if (GTK_WIDGET_REALIZED (gs)) {
 		return start_interpreter (gs);
-	} else {
+	}
+        else {
 		return 0;
 	}
 }
@@ -1594,7 +1708,7 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
  	/* default values: no dsc information available  */
         gs->structured_doc = FALSE;
  	gs->send_filename_to_gs = TRUE;
-	gs->current_page = -1;
+	gs->current_page = -2;
         gs->loaded = FALSE;
         if(*fname == '/') {
                 /* an absolute path */
@@ -1609,7 +1723,8 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
 
 	if ((gs->reading_from_pipe = (strcmp (fname, "-") == 0))) {
                 gs->send_filename_to_gs = FALSE;
-	} else {
+	}
+        else {
                 /*
                  * We need to make sure that the file is loadable/exists!
                  * otherwise we want to exit without loading new stuff...
@@ -1644,10 +1759,6 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
 				    _("Error while scanning file %s\n"), fname);
 			gtk_gs_emit_error_msg (gs, buf);
                         gtk_gs_cleanup (gs);
-                        /*
-                         * if (gdk_window_is_visible (gs->pstarget))
-			 *          gdk_window_hide (gs->pstarget);
-                        */
                         return FALSE;
                 }
 
@@ -1655,7 +1766,6 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
                     (gs->doc->epsf && gs->doc->numpages > 1)) {
                         gs->structured_doc = TRUE; 
 			gs->send_filename_to_gs = FALSE;
-			gs->pages_marked = g_new0 (gint, gs->doc->numpages);
 		}
                 
                 /* We have to set up the orientation of the document */
@@ -1716,31 +1826,25 @@ gtk_gs_next_page(GtkGS *gs)
         g_return_val_if_fail(GTK_IS_GS(gs), FALSE);
 
 	if (gs->interpreter_pid == 0) { /* no interpreter active */ 
-                 g_print ("pid = 0\n"); 
 		return FALSE;
         }
 
 	if (gs->busy) { /* interpreter is busy */
-                 g_print ("int busy\n"); 
 		return FALSE;
         }
 
 	gs->busy = TRUE;
  
-        /* g_print("Sending NEXT message to %ld\n", gs->message_window);*/
-        
 	event.xclient.type          = ClientMessage;
   	event.xclient.display       = gdk_display;
 	event.xclient.window        = gs->message_window;
 	event.xclient.message_type  = gdk_x11_atom_to_xatom(gs_class->next_atom);
 	event.xclient.format        = 32;
         
-        /* g_print ("sending event."); */
         gdk_send_xevent(gs->message_window, FALSE, 0, &event);     
  
-        /* g_print ("..sent\n"); */
-              
         gdk_flush();
+
 	return TRUE;
 }
 
@@ -1831,6 +1935,8 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
                 if (!is_interpreter_ready (gs))
                         gtk_gs_enable_interpreter (gs);
 
+                gs->current_page = page;
+
 		gtk_gs_next_page(gs);
         }
         return TRUE;
@@ -2042,9 +2148,6 @@ gtk_gs_zoom_to_fit(GtkGS *gs, gboolean fit_width)
 			new_zoom = (gfloat)gs->widget.allocation.height/(gfloat)gs->height*gs->zoom_factor;
 	}
 
-        if(ABS(new_zoom - gs->zoom_factor) < 0.001)
-		return 0.0;
-
         return new_zoom;
 }
 
@@ -2149,7 +2252,8 @@ gtk_gs_set_antialiasing(GtkGS *gs, gboolean f)
 
         gs->antialiased = f;
         gs->changed = TRUE;
-        start_interpreter(gs);
+        if(GTK_WIDGET_REALIZED(gs))
+                start_interpreter(gs);
         gtk_gs_goto_page(gs, gs->current_page);
 }
 
@@ -2226,24 +2330,6 @@ gtk_gs_get_document_page_label(GtkGS *widget, int page)
 	return NULL;
 }
 
-gint 
-gtk_gs_count_marked_pages(GtkGS *widget)
-{
-        int i;
-        int count=0;
-	g_return_val_if_fail(widget != NULL, 0);
-	g_return_val_if_fail(GTK_IS_GS(widget), 0);
-
-        if (!(widget->structured_doc && widget->doc &&
-              widget->pages_marked))
-                return 0;
-
-        for (i=0;i<widget->doc->numpages; i++)
-                if (widget->pages_marked[i])
-                        count++;
-        return count;
-}
-
 gint
 gtk_gs_get_size_index(gchar *string, GtkGSPaperSize *size)
 {
@@ -2441,3 +2527,43 @@ gtk_gs_scroll_step(GtkGS *gs, GtkScrollType direction, gboolean dowrap)
   
         return TRUE;
 }
+
+gchar *
+gtk_gs_get_postscript(GtkGS *gs, gint *pages)
+{
+        GtkGSDocSink *sink;
+        gchar *doc;
+        gboolean free_pages = FALSE;
+
+        sink = gtk_gs_doc_sink_new();
+
+        if(pages == NULL) {
+                int i, n = gtk_gs_get_page_count(gs);
+                pages = g_new0(gint, n);
+                for(i = 0; i < n; i++)
+                        pages[i] = TRUE;
+                free_pages = TRUE;
+        }
+
+        if(GTK_GS_IS_PDF(gs)) {
+                /* Save marked pages of the dsc file. The saved
+                 * file is useless without the pdf file, but can
+                 * be used to print the marked pages.
+                 * Printing only works with gs, not with a 'real'
+                 * postscript printer.
+                 * FIXME: Can we produce a real pdf file with ps2pdf?
+                 */
+                pscopydoc(sink, gs->gs_filename_dsc,
+                          gs->doc, pages);
+        }
+        else {
+                /* Use uncompressed file if necessary */
+                pscopydoc(sink, GTK_GS_GET_PS_FILE(gs), gs->doc, pages);
+        }
+        if(free_pages)
+                g_free(pages);
+
+        doc = gtk_gs_doc_sink_get_buffer(sink);
+        gtk_gs_doc_sink_free(sink);
+        return doc;
+}
